@@ -351,16 +351,16 @@ class SpyderPdb(ipyPdb):
     do_bt = do_where
 
     # --- Method defined by us to respond to ipython complete protocol
-    def do_complete(self, code, cursor_pos):
+    async def do_complete(self, code, cursor_pos):
         """
         Respond to a complete request.
         """
         if self.pdb_use_exclamation_mark:
-            return self._complete_exclamation(code, cursor_pos)
+            return await self._complete_exclamation(code, cursor_pos)
         else:
-            return self._complete_default(code, cursor_pos)
+            return await self._complete_default(code, cursor_pos)
 
-    def _complete_default(self, code, cursor_pos):
+    async def _complete_default(self, code, cursor_pos):
         """
         Respond to a complete request if not pdb_use_exclamation_mark.
         """
@@ -422,7 +422,7 @@ class SpyderPdb(ipyPdb):
                 else:
                     frame = self.curframe
                 self.shell.set_completer_frame(frame)
-            result = self.shell.kernel._do_complete(code, cursor_pos)
+            result = await self.shell.kernel._do_complete(code, cursor_pos)
             # Reset frame
             self.shell.set_completer_frame()
             # If there is no Pdb results to merge, return the result
@@ -450,7 +450,7 @@ class SpyderPdb(ipyPdb):
                 'metadata': {},
                 'status': 'ok'}
 
-    def _complete_exclamation(self, code, cursor_pos):
+    async def _complete_exclamation(self, code, cursor_pos):
         """
         Respond to a complete request if pdb_use_exclamation_mark.
         """
@@ -529,7 +529,7 @@ class SpyderPdb(ipyPdb):
             else:
                 frame = self.curframe
             self.shell.set_completer_frame(frame)
-        result = self.shell.kernel._do_complete(code, cursor_pos)
+        result = await self.shell.kernel._do_complete(code, cursor_pos)
         # Reset frame
         self.shell.set_completer_frame()
         return result
@@ -687,7 +687,12 @@ class SpyderPdb(ipyPdb):
         stop = None
         while not stop:
             if self.cmdqueue:
-                line = self.cmdqueue.pop(0)
+                # Anything available in cmdqueue is a Pdb command, so we need
+                # to process it as such.
+                # Fixes spyder-ide/spyder#22500
+                line = (
+                    "!" if self.pdb_use_exclamation_mark else ""
+                ) + self.cmdqueue.pop(0)
             else:
                 try:
                     line = self.cmd_input(self.prompt)
